from __future__ import absolute_import, unicode_literals

import logging

from django.core.exceptions import ImproperlyConfigured
from django.test import TestCase
from django.test.utils import override_settings
from django.utils.unittest import skipUnless

import djhipchat
from djhipchat.logger import HipChatHandler
from djhipchat import get_backend

try:
    from djhipchat.tasks import send_message as send_message_task
except (ImportError, ImproperlyConfigured):
    send_message_task = None


def _backend(name):
    return 'djhipchat.backends.%s.HipChatBackend' % name


class BackendTest(TestCase):
    def test_backend_errors(self):
        self.assertRaises(ImproperlyConfigured,
                          lambda: get_backend('djhipchat.foo.BadModule'))
        self.assertRaises(ImproperlyConfigured,
                          lambda: get_backend(
                              'djhipchat.backends.dummy.BadClass'))

    @override_settings(HIPCHAT_BACKEND=_backend('dummy'))
    def test_dummy(self):
        backend = get_backend(_backend('dummy'))
        self.assertIsNotNone(backend)

        djhipchat.sent_messages = []
        djhipchat.send_message('1234', 'dummy')
        self.assertEqual([], djhipchat.sent_messages)

    @override_settings(HIPCHAT_BACKEND=_backend('locmem'))
    def test_locmem(self):
        backend = get_backend(_backend('locmem'))
        self.assertIsNotNone(backend)

        djhipchat.sent_messages = []
        djhipchat.send_message('1234', 'locmem test', sender='TestFrom')
        # (room_id, message, sender, message_format, notify, color)
        expected = {
            'room_id': '1234',
            'message': 'locmem test',
            'from': 'TestFrom',
            'format': 'json',
            'message_format': 'html',
            'notify': 0,
            'color': 'yellow'
        }
        self.assertEqual([expected], djhipchat.sent_messages)

    @override_settings(HIPCHAT_BACKEND=_backend('locmem'))
    def test_default_sender(self):
        djhipchat.sent_messages = []
        with override_settings(HIPCHAT_DEFAULT_SENDER='TestSender'):
            djhipchat.send_message('1234', 'test sender')
            self.assertEqual('TestSender', djhipchat.sent_messages[0]['from'])

        djhipchat.sent_messages = []
        with override_settings(HIPCHAT_DEFAULT_SENDER=None):
            djhipchat.send_message('1234', 'test sender')
            self.assertEqual('Django', djhipchat.sent_messages[0]['from'])


@skipUnless(send_message_task, "requires celery")
@override_settings(HIPCHAT_BACKEND=_backend('celery'),
                   HIPCHAT_CELERY_BACKEND=_backend('locmem'))
class CeleryTest(TestCase):
    def test_backend(self):
        backend = get_backend(_backend('celery'))
        self.assertIsNotNone(backend)

        djhipchat.sent_messages = []
        djhipchat.send_message('1234', 'dummy', sender='TestFrom')
        expected = {
            'room_id': '1234',
            'message': 'dummy',
            'from': 'TestFrom',
            'format': 'json',
            'message_format': 'html',
            'notify': 0,
            'color': 'yellow'
        }
        self.assertEqual([expected], djhipchat.sent_messages)


@override_settings(HIPCHAT_BACKEND=_backend('locmem'))
class LoggerTest(TestCase):
    def test_logger_basic(self):
        """
        Very simple test of the logger and color selection.
        """
        logger = logging.getLogger("logger.test")
        logger.addHandler(HipChatHandler("1234", sender='Test', colors={
            'ERROR': 'red',
            'INFO': 'grey'
        }))
        logger.setLevel(logging.INFO)

        djhipchat.sent_messages = []
        logger.info("Message: %d", 5)
        logger.error("Error: %s", "hello")

        expected1 = {
            'room_id': '1234',
            'message': 'Message: 5',
            'from': 'Test',
            'format': 'json',
            'message_format': 'text',
            'notify': 0,
            'color': 'grey'
        }
        expected2 = {
            'room_id': '1234',
            'message': 'Error: hello',
            'from': 'Test',
            'format': 'json',
            'message_format': 'text',
            'notify': 0,
            'color': 'red'
        }
        self.assertEqual([expected1, expected2], djhipchat.sent_messages)



from __future__ import absolute_import, unicode_literals

from optparse import make_option

from django.core.management.base import BaseCommand, CommandError


class Command(BaseCommand):
    args = '<room_id> <message>'
    help = 'Sends a message to a HipChat room.'
    option_list = BaseCommand.option_list + (
        make_option(
            '-f', '--from',
            action='store',
            dest='sender',
            help='The sender of the message'),
        make_option(
            '--format',
            action='store',
            default='html',
            dest='message_format',
            choices=('text', 'html'),
            help='The format of the message, text or html (default html)'),
        make_option(
            '-n', '--notify',
            action='store_true',
            default=False,
            dest='notify',
            help='Causes a notification to appear in the room'),
        make_option(
            '-c', '--color',
            action='store',
            default='yellow',
            dest='color',
            choices=("yellow", "red", "green", "purple", "gray", "random"),
            help='Color to use for the notification.')
    )
    help = 'Sends a message to a HipChat room.'

    def handle(self, *args, **options):
        if len(args) != 2:
            raise CommandError('Incorrect number of arguments: hipchat ' +
                               self.args)

        kwargs = {
            'sender': options['sender'],
            'message_format': options['message_format'],
            'notify': options['notify'],
            'color': options['color']
        }

        from djhipchat import send_message
        result = send_message(*args, **kwargs)

        if 'error' in result:
            raise CommandError(result['error']['message'])



from __future__ import unicode_literals


class HipChatBackendBase(object):
    def get_payload(self, room_id, message, sender,
                    message_format='html',
                    notify=False,
                    color='yellow'):
        data = {
            'room_id': room_id,
            'from': sender,
            'message': message,
            'message_format': message_format,
            'notify': 1 if notify else 0,
            'color': color,
            'format': 'json'
        }
        return data



from __future__ import absolute_import, unicode_literals

import djhipchat
from djhipchat.backends.base import HipChatBackendBase


class HipChatBackend(HipChatBackendBase):
    def __init__(self, *args, **kwargs):
        super(HipChatBackend, self).__init__(*args, **kwargs)
        if not hasattr(djhipchat, 'sent_messages'):
            djhipchat.sent_messages = []

    def send_message(self, room_id, message, sender,
                     message_format='html', notify=False, color='yellow'):
        data = self.get_payload(room_id, message, sender,
                                message_format, notify, color)

        djhipchat.sent_messages.append(data)



from __future__ import absolute_import, unicode_literals

from djhipchat.backends.base import HipChatBackendBase


class HipChatBackend(HipChatBackendBase):
    """
    Dummy backend. Doesn't do anything.
    """
    def __init__(self, *args, **kwargs):
        super(HipChatBackend, self).__init__(*args, **kwargs)

    def send_message(self, room_id, message, sender,
                     message_format='html', notify=False, color='yellow'):
        pass



from __future__ import absolute_import, unicode_literals

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

try:
    from celery.task import task
except ImportError:
    raise ImproperlyConfigured("Celery required for celery backend")

from djhipchat import get_backend


@task(ignore_result=True)
def send_message(*args, **kwargs):
    try:
        backend = get_backend(getattr(settings, 'HIPCHAT_CELERY_BACKEND'))
    except AttributeError:
        raise ImproperlyConfigured(
            "You need to specify HIPCHAT_CELERY_BACKEND")

    backend.send_message(*args, **kwargs)



"""
Custom log handler for posting log messages to HipChat.

The API documentation is available at:
    https://www.hipchat.com/docs/api/method/rooms/message
The room id can be found by going to:
    https://{{your-account}}.hipchat.com/rooms/ids
The tokens can be set at https://{{your-account}}.hipchat.com/admin/api

Inspired by: https://gist.github.com/hugorodgerbrown/3176710
"""
from __future__ import absolute_import, unicode_literals

from logging import Handler

from djhipchat import send_message


class HipChatHandler(Handler):
    """
    Log handler used to send notifications to HipChat
    """
    def __init__(self, room_id, colors=None, **kwargs):
        """
        Arguments match those of send_message, with the exception of:
        :param colors: A dict of level: color pairs used to override
                    the default color.
        """
        super(HipChatHandler, self).__init__()
        self.room_id = room_id
        self.kwargs = kwargs
        self.colors = colors or {}

    def emit(self, record):
        """
        Sends the record info to HipChat
        """
        kwargs = {
            'room_id': self.room_id,
            'message': record.getMessage(),
            'message_format': 'text'
        }
        kwargs.update(self.kwargs)
        if record.levelname in self.colors:
            kwargs['color'] = self.colors[record.levelname]
        send_message(**kwargs)



#!/usr/bin/env python
import os
import codecs
from setuptools import setup

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

if os.path.exists('README.rst'):
    long_description = codecs.open('README.rst', 'r', 'utf-8').read()
else:
    long_description = 'See https://github.com/paulcwatts/djhipchat2'

setup(
    name='djhipchat2',
    version='0.5.1',
    packages=[
        'djhipchat',
        'djhipchat.backends',
        'djhipchat.management',
        'djhipchat.management.commands'
    ],
    url='https://github.com/paulcwatts/djhipchat2',
    license='BSD',
    author='Paul Watts',
    author_email='paulcwatts@gmail.com',
    description='The most complete and configurable HipChat library for Django.',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7'
    ],
    long_description=long_description
)



from __future__ import absolute_import, unicode_literals

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils.importlib import import_module


HIPCHAT_BACKEND = getattr(settings, 'HIPCHAT_BACKEND',
                          'djhipchat.backends.locmem.HipChatBackend')


def get_backend(backend=None, **kwargs):
    path = backend or settings.HIPCHAT_BACKEND
    try:
        mod_name, klass_name = path.rsplit('.', 1)
        mod = import_module(mod_name)
    except ImportError as e:
        raise ImproperlyConfigured(
            ('Error importing HipChat backend module %s: "%s"' %
             (mod_name, e)))
    try:
        klass = getattr(mod, klass_name)
    except AttributeError:
        raise ImproperlyConfigured(('Module "%s" does not define a '
                                    '"%s" class' % (mod_name, klass_name)))
    return klass(**kwargs)


def send_message(room_id, message, sender=None,
                 message_format='html', notify=False,
                 color='yellow'):
    """
    Sends a message to HipChat.

    :param room_id: The ID of the Room to send to.
    :param sender: The text name of the sender.
    :param message: The text or HTML of the message.
    :param message_format: 'text' or 'html'.
    :param notify: Whether to trigger a notification for users in the room.
    :param color: The color of the message.
    """
    sender = (sender or
              getattr(settings, 'HIPCHAT_DEFAULT_SENDER', '') or
              'Django')
    return get_backend().send_message(room_id, message, sender,
                                      message_format, notify, color)



from __future__ import absolute_import, unicode_literals

import requests

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from djhipchat.backends.base import HipChatBackendBase


HIPCHAT_API_URL = 'https://api.hipchat.com/v1/rooms/message'


class HipChatBackend(HipChatBackendBase):
    def __init__(self, *args, **kwargs):
        super(HipChatBackend, self).__init__(*args, **kwargs)

    def send_message(self, room_id, message, sender,
                     message_format='html', notify=False, color='yellow'):
        token = getattr(settings, 'HIPCHAT_API_TOKEN', '')
        if not token:
            raise ImproperlyConfigured("HipChat API token not set")

        data = self.get_payload(room_id, message, sender,
                                message_format, notify, color)
        data['auth_token'] = token

        result = requests.post(HIPCHAT_API_URL, data=data)
        if 'error' in result:
            raise ValueError(result['error']['message'])



from __future__ import absolute_import, unicode_literals

from djhipchat.backends.base import HipChatBackendBase
from djhipchat.tasks import send_message


class HipChatBackend(HipChatBackendBase):
    def __init__(self, *args, **kwargs):
        super(HipChatBackend, self).__init__(*args, **kwargs)

    def send_message(self, room_id, message, sender,
                     message_format='html', notify=False, color='yellow'):
        send_message.delay(room_id, message, sender,
                           message_format, notify, color)
